"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.watchPostGraphileSchema = exports.createPostGraphileSchema = exports.postGraphileClassicIdsInflection = exports.postGraphileInflection = exports.postGraphileClassicIdsOverrides = exports.postGraphileBaseOverrides = exports.watchPostGraphQLSchema = exports.createPostGraphQLSchema = exports.postGraphQLClassicIdsInflection = exports.postGraphQLInflection = exports.postGraphQLClassicIdsOverrides = exports.postGraphQLBaseOverrides = exports.inflections = undefined;

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _graphileBuild = require("graphile-build");

var _graphileBuildPg = require("graphile-build-pg");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const ensureValidPlugins = (name, arr) => {
  if (!Array.isArray(arr)) {
    throw new Error(`Option '${name}' should be an array`);
  }
  for (let i = 0, l = arr.length; i < l; i++) {
    const fn = arr[i];
    if (typeof fn !== "function") {
      throw new Error(`Option '${name}' should be an array of functions, found '${typeof fn}' at index ${i}`);
    }
  }
};
exports.inflections = _graphileBuildPg.inflections;
const postGraphQLBaseOverrides = exports.postGraphQLBaseOverrides = {
  enumName(value) {
    return _graphileBuildPg.inflections.defaultUtils.constantCase(_graphileBuildPg.inflections.defaultInflection.enumName(value));
  }
};

const postGraphQLClassicIdsOverrides = exports.postGraphQLClassicIdsOverrides = {
  column(name, _table, _schema) {
    return name === "id" ? "rowId" : _graphileBuildPg.inflections.defaultUtils.camelCase(name);
  }
};

const postGraphQLInflection = exports.postGraphQLInflection = _graphileBuildPg.inflections.newInflector(postGraphQLBaseOverrides);
const postGraphQLClassicIdsInflection = exports.postGraphQLClassicIdsInflection = _graphileBuildPg.inflections.newInflector((0, _assign2.default)({}, postGraphQLBaseOverrides, postGraphQLClassicIdsOverrides));

const awaitKeys = async obj => {
  const result = {};
  for (const k in obj) {
    result[k] = await obj[k];
  }
  return result;
};

const getPostGraphQLBuilder = async (pgConfig, schemas, options = {}) => {
  const { dynamicJson, classicIds, nodeIdFieldName } = options;
  const {
    replaceAllPlugins,
    appendPlugins = [],
    prependPlugins = [],
    skipPlugins = [],
    jwtPgTypeIdentifier,
    jwtSecret,
    disableDefaultMutations,
    graphileBuildOptions,
    graphqlBuildOptions, // DEPRECATED!
    inflector,
    pgColumnFilter,
    viewUniqueKey,
    enableTags = true,
    readCache,
    writeCache,
    setWriteCacheCallback
  } = options;
  if (replaceAllPlugins) {
    ensureValidPlugins("replaceAllPlugins", replaceAllPlugins);
    if (prependPlugins && prependPlugins.length || appendPlugins && appendPlugins.length) {
      throw new Error("When using 'replaceAllPlugins' you must not specify 'appendPlugins'/'prependPlugins'");
    }
  }
  if (readCache && writeCache) {
    throw new Error("Use `readCache` or `writeCache` - not both.");
  }

  let persistentMemoizeWithKey = undefined; // NOT null, otherwise it won't default correctly.
  let memoizeCache = {};

  if (readCache) {
    memoizeCache = JSON.parse((await new _promise2.default((resolve, reject) => {
      _fs2.default.readFile(readCache, "utf8", (err, data) => {
        if (err) {
          reject(err);
        } else {
          resolve(data);
        }
      });
    })));
  }
  if (readCache || writeCache) {
    persistentMemoizeWithKey = (key, fn) => {
      if (!(key in memoizeCache)) {
        if (readCache) {
          throw new Error(`Expected cache to contain key: ${key}`);
        }
        memoizeCache[key] = fn();
        if (memoizeCache[key] === undefined) {
          throw new Error(`Cannot memoize 'undefined' - use 'null' instead`);
        }
      }
      return memoizeCache[key];
    };
  }

  if (writeCache && setWriteCacheCallback) {
    setWriteCacheCallback(() => awaitKeys(memoizeCache).then(obj => new _promise2.default((resolve, reject) => {
      _fs2.default.writeFile(writeCache, (0, _stringify2.default)(obj), err => {
        memoizeCache = {};
        if (err) {
          reject(err);
        } else {
          resolve();
        }
      });
    })));
  } else if (writeCache) {
    throw new Error("Cannot write cache without 'setWriteCacheCallback'");
  } else if (setWriteCacheCallback) {
    setWriteCacheCallback(() => _promise2.default.resolve());
  }

  ensureValidPlugins("prependPlugins", prependPlugins);
  ensureValidPlugins("appendPlugins", appendPlugins);
  ensureValidPlugins("skipPlugins", skipPlugins);
  return (0, _graphileBuild.getBuilder)((replaceAllPlugins ? [...prependPlugins, ...replaceAllPlugins, ...appendPlugins] : [...prependPlugins, ..._graphileBuild.defaultPlugins, ..._graphileBuildPg.defaultPlugins, ...appendPlugins]).filter(p => skipPlugins.indexOf(p) === -1), (0, _assign2.default)({
    pgConfig: pgConfig,
    pgSchemas: Array.isArray(schemas) ? schemas : [schemas],
    pgExtendedTypes: !!dynamicJson,
    pgColumnFilter: pgColumnFilter || (() => true),
    pgInflection: inflector || (classicIds ? postGraphQLClassicIdsInflection : postGraphQLInflection),
    nodeIdFieldName: nodeIdFieldName || (classicIds ? "id" : "nodeId"),
    pgJwtTypeIdentifier: jwtPgTypeIdentifier,
    pgJwtSecret: jwtSecret,
    pgDisableDefaultMutations: disableDefaultMutations,
    pgViewUniqueKey: viewUniqueKey,
    pgEnableTags: enableTags,
    persistentMemoizeWithKey
  }, graphileBuildOptions, graphqlBuildOptions // DEPRECATED!
  ));
};

function abort(e) {
  /* eslint-disable no-console */
  console.error("Error occured whilst writing cache");
  console.error(e);
  process.exit(1);
  /* eslint-enable */
}

const createPostGraphQLSchema = exports.createPostGraphQLSchema = async (pgConfig, schemas, options = {}) => {
  let writeCache;
  const builder = await getPostGraphQLBuilder(pgConfig, schemas, (0, _assign2.default)({}, options, {
    setWriteCacheCallback(fn) {
      writeCache = fn;
    }
  }));
  const schema = builder.buildSchema();
  if (writeCache) writeCache().catch(abort);
  return schema;
};

/*
 * Unless an error occurs, `onNewSchema` is guaranteed to be called before this promise resolves
 */
const watchPostGraphQLSchema = exports.watchPostGraphQLSchema = async (pgConfig, schemas, options = {}, onNewSchema) => {
  if (typeof onNewSchema !== "function") {
    throw new Error("You cannot call watchPostGraphQLSchema without a function to pass new schemas to");
  }
  if (options.readCache) {
    throw new Error("Using readCache in watch mode does not make sense.");
  }
  let writeCache;
  const builder = await getPostGraphQLBuilder(pgConfig, schemas, (0, _assign2.default)({}, options, {
    setWriteCacheCallback(fn) {
      writeCache = fn;
    }
  }));
  let released = false;
  function handleNewSchema(...args) {
    if (writeCache) writeCache().catch(abort);
    onNewSchema(...args);
  }
  await builder.watchSchema(handleNewSchema);

  return async function release() {
    if (released) return;
    released = true;
    await builder.unwatchSchema();
  };
};

const postGraphileBaseOverrides = exports.postGraphileBaseOverrides = postGraphQLBaseOverrides;
const postGraphileClassicIdsOverrides = exports.postGraphileClassicIdsOverrides = postGraphQLClassicIdsOverrides;
const postGraphileInflection = exports.postGraphileInflection = postGraphQLInflection;
const postGraphileClassicIdsInflection = exports.postGraphileClassicIdsInflection = postGraphQLClassicIdsInflection;
const createPostGraphileSchema = exports.createPostGraphileSchema = createPostGraphQLSchema;
const watchPostGraphileSchema = exports.watchPostGraphileSchema = watchPostGraphQLSchema;
//# sourceMappingURL=index.js.map